package java.rmi;

import java.net.URI;
import java.net.URISyntaxException;
import java.rmi.registry.LocateRegistry;
import java.net.MalformedURLException;
import java.rmi.registry.Registry;

public final class Naming
{
    private Naming() {
    }
    
    public static Remote lookup(String s) throws NotBoundException, MalformedURLException, RemoteException {
        Naming.ParsedNamingURL url = parseURL(s);
        Registry registry = getRegistry(url);
        if (url.name == null) {
            return registry;
        }
        return registry.lookup(url.name);
    }
    
    public static void bind(String s, Remote remote) throws AlreadyBoundException, MalformedURLException, RemoteException {
        Naming.ParsedNamingURL url = parseURL(s);
        Registry registry = getRegistry(url);
        if (remote == null) {
            throw new NullPointerException("cannot bind to null");
        }
        registry.bind(url.name, remote);
        return;
    }
    
    public static void unbind(String s) throws RemoteException, NotBoundException, MalformedURLException {
        Naming.ParsedNamingURL url = parseURL(s);
        getRegistry(url).unbind(url.name);
    }
    
    public static void rebind(String s, Remote remote) throws RemoteException, MalformedURLException {
        Naming.ParsedNamingURL url = parseURL(s);
        Registry registry = getRegistry(url);
        if (remote == null) {
            throw new NullPointerException("cannot bind to null");
        }
        registry.rebind(url.name, remote);
        return;
    }
    
    public static String[] list(String s) throws RemoteException, MalformedURLException {
        Naming.ParsedNamingURL url = parseURL(s);
        Registry registry = getRegistry(url);
        String str = "";
        if (url.port > 0 || !url.host.equals("")) {
            str = str + "//" + url.host;
        }
        if (url.port > 0) {
            str = str + ":" + url.port;
        }
        String string = str + "/";
        String[] list = registry.list();
        for (int i = 0; i < list.length; ++i) {
            list[i] = string + list[i];
        }
        return list;
    }
    
    private static Registry getRegistry(Naming.ParsedNamingURL parsedNamingURL) throws RemoteException {
        return LocateRegistry.getRegistry(parsedNamingURL.host, parsedNamingURL.port);
    }
    
    private static Naming.ParsedNamingURL parseURL(String str) throws MalformedURLException {
        try {
            return intParseURL(str);
        }
        catch (URISyntaxException cause) {
            MalformedURLException ex = new MalformedURLException("invalid URL String: " + str);
            ex.initCause(cause);
            int index = str.indexOf(58);
            int index2 = str.indexOf("//:");
            if (index2 < 0) {
                throw ex;
            }
            if (index2 == 0 || (index > 0 && index2 == index + 1)) {
                int n = index2 + 2;
                String string = str.substring(0, n) + "localhost" + str.substring(n);
                try {
                    return intParseURL(string);
                }
                catch (URISyntaxException ex2) {
                    throw ex;
                }
                catch (MalformedURLException ex3) {
                    throw ex3;
                }
            }
            else {
                throw ex;
            }
        }
    }
    
    private static Naming.ParsedNamingURL intParseURL(String s) throws MalformedURLException, URISyntaxException {
        URI uri = new URI(s);
        if (uri.isOpaque()) {
            throw new MalformedURLException("not a hierarchical URL: " + s);
        }
        if (uri.getFragment() != null) {
            throw new MalformedURLException("invalid character, '#', in URL name: " + s);
        }
        if (uri.getQuery() != null) {
            throw new MalformedURLException("invalid character, '?', in URL name: " + s);
        }
        if (uri.getUserInfo() != null) {
            throw new MalformedURLException("invalid character, '@', in URL host: " + s);
        }
        String scheme = uri.getScheme();
        if (scheme != null && !scheme.equals("rmi")) {
            throw new MalformedURLException("invalid URL scheme: " + s);
        }
        String s2 = uri.getPath();
        if (s2 != null) {
            if (s2.startsWith("/")) {
                s2 = s2.substring(1);
            }
            if (s2.length() == 0) {
                s2 = null;
            }
        }
        String host = uri.getHost();
        if (host == null) {
            host = "";
            try {
                uri.parseServerAuthority();
            }
            catch (URISyntaxException ex) {
                String authority = uri.getAuthority();
                if (authority != null && authority.startsWith(":")) {
                    String string = "localhost" + authority;
                    try {
                        uri = new URI(null, string, null, null, null);
                        uri.parseServerAuthority();
                    }
                    catch (URISyntaxException ex2) {
                        throw new MalformedURLException("invalid authority: " + s);
                    }
                }
                else {
                    throw new MalformedURLException("invalid authority: " + s);
                }
            }
        }
        int port = uri.getPort();
        if (port == -1) {
            port = 1099;
        }
        return new ParsedNamingURL(host, port, s2);
    }
    
    private static class ParsedNamingURL
    {
        String host;
        int port;
        String name;
        
        ParsedNamingURL(String host, int port, String name) {
            this.host = host;
            this.port = port;
            this.name = name;
        }
    }
}
